import java.util.Random;
import java.util.Arrays;

/**
   Define and demonstrate an iterative bubble
   sort algorithm on integer arrays.
*/
public class BubbleSort
{
   /**
      This main() method initializes an array with random values
      and then calls the bubble sort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(25, 100);
      System.out.println( Arrays.toString(a) );

      sort(a);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using iterative bubble sort.

      @param a  the array to sort
   */
   public static void sort(int[] a)
   {
      for (int i = 0; i < a.length - 1; ++i)
      {
         for (int j = 0; j < a.length - 1 - i; ++j)
         {
            if ( a[j] > a[j + 1] )
            {
               swap(a, j, j + 1);
            }
         }
      }
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   public static void swap(int[] a, int i, int j)
   {
      final int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
