import java.util.Random;
import java.util.Arrays;

/**
   Define and demonstrate an iterative insertion
   sort algorithm on integer arrays.
*/
public class InsertionSort_Loop
{
   /**
      This main() method initializes an array with random values
      and then calls the insertion sort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(25, 100);
      System.out.println( Arrays.toString(a) );

      sort(a);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using iterative insertion sort.

      @param a  the array to sort
   */
   public static void sort(int[] a)
   {
      for (int i = 1; i < a.length; ++i)
      {
         final int next = a[i];
         // Move all larger elements up.
         int j = i;
         while (j > 0 && a[j - 1] > next)
         {
            a[j] = a[j - 1];
            j--;
         }
         // Insert the element.
         a[j] = next;
      }
   }
}
