import java.util.Random;
import java.util.Arrays;

/**
   Define and demonstrate a recursive insertion
   sort algorithm on integer arrays.
*/
public class InsertionSort_Recursion_v2
{
   /**
      This main() method initializes an array with random values
      and then calls the insertion sort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(25, 100);
      System.out.println( Arrays.toString(a) );

      sort(a, 1);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using recursive insertion sort.

      This version sorts the array in the direction
      from the beginning towards the end.

      Assumption: The array elements from
      index 0 up to index front-1 are sorted.

      This method inserts the array element
      at index front into the correct, sorted,
      position in the sub-array from index 0 to index front.

      Then this method recursively sorts the subarray
      from index 0 to index front+1.

      @param a      the array to sort
      @param front  the length of the initial sorted subarray

      return with the elements of a from index front to the last index are sorted
   */
   public static void sort(int[] a, int front)
   {
      if ( front >= a.length ) // base case
      {
         return;
      }
      else // recursive case
      {
         insert(a, front);
         sort(a, front + 1);  // recursion
      }
   }


   /**
      Assumption: The array elements from
      index 0 up to index frontIndex - 1
      are sorted.

      Insert the array element at index frontIndex into
      its proper (sorted) location in the elements between
      index 0 and index frontIndex.

      @param a           the array to sort
      @param frontIndex  the index of the element to insert

      return with the elements of a in the range
             a[0] ... a[frontIndex] sorted
   */
   private static void insert(int[] a, int frontIndex)
   {
      if ( frontIndex == 0 ) // a base case
      {
         return;
      }
      else if ( a[frontIndex] >= a[frontIndex - 1] ) // another base case
      {
         return;
      }
      else // recursive case
      {
         swap(a, frontIndex - 1, frontIndex);
         insert(a, frontIndex - 1); // recursion
      }
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   public static void swap(int[] a, int i, int j)
   {
      final int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
