import java.util.Arrays;
import java.util.Random;

/**
   This program measures the runtime of the merge
   sort algorithm on a sequence of growing arrays.

   For an array of length n, the runtime of merge
   sort is approximately n*lg(n),
       T(n) ~ n*lg(n)
   This is sometimes written as O(n*lg(n)) where
   lg is the logarithm to base 2 function.

   This is quite a bit faster than the O(n^2) of
   selection sort and insertion sort. For large
   numbers n, compare the values of
      n^2
   and
      n*lg(n).
   For example, for n = 1024 = 2^10,
      n^2 = 2^20 = 1,048,576
   but
      n*lg(n) = 1024 * 10 = 10,240.
*/
public class MergeSort_Profile
{
   public static void main(String[] args)
   {
      final int size = 1_000_000; // starting size for the arrays
      final int iterations = 20;  // how many times to increase the array size

      System.out.println("Merge sort.");

      for (int i = 1; i <= iterations; ++i)
      {
         final int[] array = MergeSort.randomIntArray(i*size, 1_000_000);

         final long startTime = System.currentTimeMillis();
            MergeSort.sort(array);
         final long stopTime = System.currentTimeMillis();
         System.out.println("For n = " + (i*size) + ", time = " + (stopTime - startTime));
      }
   }
}
