import java.util.Arrays;
import java.util.Random;

/**
   Define and demonstrate a recursive selection
   sort algorithm on integer arrays.
*/
public class SelectionSort_Recursion_v1
{
   /**
      This main() method initializes an array with random values
      and then calls the insertion sort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(25, 100);
      System.out.println( Arrays.toString(a) );

      sort(a, a.length);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using recursive selection sort.

      This version sorts the array in the direction
      from the end towards the front.

      This method first finds the largest element of the array
      and moves that element to the end of the array. Then it
      recursively sorts the subarray that is one item shorter.

      @param a      the array to sort
      @param front  the length of the initial subarray to be sorted

      return with the elements of a from index 0 to index front-1 sorted
   */
   public static void sort(int[] a, int front)
   {
      if ( front == 1 ) // base case
      {
         return;
      }
      else // recursive case
      {
         final int maxPos = maximumPosition(a, front);
         swap(a, maxPos, front-1);
         sort(a, front - 1);  // recursion
      }
   }


   /**
      Find the largest element in an initial range of the array.

      @param a        the array to sort
      @param toIndex  the length of the initial subarry to be searched
      @return the position of the largest element in the
              range a[0] . . . a[toIndex - 1]
   */
   private static int maximumPosition(int[] a, int toIndex)
   {
      int maxPosition = 0; // initial guess of where the max is positioned
      for (int i = 1; i < toIndex; ++i)
      {
         if (a[i] > a[maxPosition])
         {
            maxPosition = i; // update the guess
         }
      }
      return maxPosition;
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   public static void swap(int[] a, int i, int j)
   {
      final int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
